-------------------------------------------------------------------------------
--
--    File Name:  clock.vhd
--      Version:  1.0
--         Date:  January 24, 2001
--  Description:  Clock management module
-- Dependencies:  none
--
--      Company:  Xilinx
--
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER.
--
--                Copyright (c) 2001 Xilinx, Inc.
--                All rights reserved
--
-------------------------------------------------------------------------------
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_unsigned.all;
use IEEE.std_logic_arith.all;

-- synthesis translate_off
library unisim;
use unisim.all;
-- synthesis translate_on

entity clock is
  port (
    clkin    : in  std_logic;
    rst      : in  std_logic;
    clk2x    : out std_logic;
    clk      : out std_logic;
    lock     : out std_logic;
    phs_l    : out std_logic;
    clk2xcam : out std_logic
    );
end clock;

architecture clock_arch of clock is

signal clk2xi, clk1xi, clk_bufg, clk2x_bufg, clk2xfbi: std_logic;

signal clk2xfb_bufg : std_logic;

-- declare the DLL component interface

component CLKDLL
      port (
        CLKIN, CLKFB, RST : in STD_LOGIC;
        CLK0, CLK90, CLK180, CLK270, CLK2X, CLKDV, LOCKED : out std_logic);
end component; 

-- declare the Global Buffer, BUFG, component interface
component BUFG
      port (I : in STD_LOGIC; O : out std_logic);
end component;
 

begin
-- instantiation of DLL for creation of clk2x and deskew of clk

dll1 : CLKDLL port map (
  CLKIN  => clkin,
  CLK0   => clk1xi,
  CLKFB  => clk_bufg,
  RST    => rst,
  CLK2X  => clk2xi,
  LOCKED => lock,
  CLK270 => phs_l);

clk1xbuf : BUFG port map (
  I => clk1xi,
  O => clk_bufg);

clk2xbuf : BUFG port map (
  I => clk2xi,
  O => clk2x_bufg);

-- note this extra bufg inserted to satifify simulation model delays
clk2xfbbuf : BUFG port map (
  I => clk2xfbi,
  O => clk2xfb_bufg);

dll2 : CLKDLL port map (
  CLKIN => clk2x_bufg,
  CLK90 => clk2xcam,
  CLK2X => clk2xfbi,
  CLKFB => clk2xfb_bufg,
  RST   => rst);

-- assignment of internal signals to output ports
clk <= clk_bufg;
clk2x <= clk2x_bufg;
end clock_arch;
