-------------------------------------------------------------------------------
--
--    File Name:  top.vhd
--      Version:  1.0
--         Date:  January 24, 2001
--  Description:  Top Level design for interface with Lara Networks CAM
-- Dependencies:  init.vhd, clock.vhd, rw.vhd, search.vhd
--
--      Company:  Xilinx
--
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER.
--
--                Copyright (c) 2000 Xilinx, Inc.
--                All rights reserved
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;

-- synthesis translate_off
library unisim;
use unisim.all;
-- synthesis translate_on

entity top is
  
  port (
    lock                : out std_logic;
    clkin               : in    std_logic;
    rst                 : in    std_logic;
    dllrst              : in    std_logic;
    user_command        : in    std_logic_vector(5 downto 0);
    rw_in               : in    std_logic_vector(67 downto 0);
    search_in           : in    std_logic_vector(67 downto 0);
    ack                 : in    std_logic;  -- Input from CAM
    ssf                 : in    std_logic;  -- Input from CAM
    ssv                 : in    std_logic;  -- Input from CAM
    eot                 : in    std_logic;  -- Input from CAM
    start               : in    std_logic;
    gmr_index           : in    std_logic_vector(2 downto 0);
    id                  : in    std_logic_vector(4 downto 0);
    sadr                : in    std_logic_vector(21 downto 0);
    sadr_index          : in    std_logic_vector(2 downto 0);
    ssr_reg_index       : in    std_logic_vector(2 downto 0);
    comparand_reg_index : in    std_logic_vector(3 downto 0);  -- Inputs from User Side
    last                : in   std_logic;

    dq_out              : inout std_logic_vector(67 downto 0);  -- Bidirectional signals between Virtex/E device and CAM
    search_out          : out   std_logic_vector(21 downto 0);
    match               : out   std_logic;
    match_valid         : out   std_logic;
    ready               : out   std_logic;  -- Output to User Side
    id_enable           : out   std_logic;  -- Output to User Side
    rw_out              : out   std_logic_vector(67 downto 0);
    cmdv_out            : out   std_logic;
    cmd_out             : out   std_logic_vector(8 downto 0);
    clk2xcam            : out   std_logic;
    phs_l               : out   std_logic);

end top;

architecture top_arch of top is
-- here is the component list for the other files
-- declare the Global Input Buffer, IBUFG, component interface
component IBUFG
      port (I : in STD_LOGIC; O : out std_logic);
end component;

component clock is
  port (
    clkin    : in  std_logic;
    rst      : in  std_logic;
    clk2x    : out std_logic;
    clk      : out std_logic;
    lock     : out std_logic;
    phs_l    : out std_logic;
    clk2xcam : out std_logic
    );
end component clock;

component init is
  generic (
    datawidth  : std_logic_vector(1 downto 0);
    cascade_depth : std_logic_vector(4 downto 0);
    tlsz : std_logic_vector(1 downto 0)); 

  port (
    clk       : in  std_logic;
    rst       : in  std_logic;
    start     : in  std_logic;
    last      : in  std_logic;
    id        : in  std_logic_vector(4 downto 0);
    cmmd      : out std_logic_vector(8 downto 0);
    dq        : out std_logic_vector (67 downto 0);
    cmmdv     : out std_logic;
    ready_i   : out std_logic;
    i_tr_dq   : out std_logic;
    i_tr_cmdv : out std_logic;
    i_tr_cmd  : out std_logic;
    id_enable : out std_logic);

end component init;

component rw is

  port (
    clk          : in  std_logic;
    rst          : in  std_logic;
    eot_in       : in  std_logic;
    ack          : in  std_logic;
    user_command : in  std_logic_vector(5 downto 0);
    data_back    : in  std_logic_vector(67 downto 0);
    gmri         : in  std_logic_vector(2 downto 0);
    id           : in  std_logic_vector(4 downto 0);
    data         : in std_logic_vector(67 downto 0);
    cmmdv        : out std_logic;
    rw_tr_dq     : out std_logic;
    rw_tr_cmdv   : out std_logic;
    rw_tr_cmd    : out std_logic;
    ready_rw     : out std_logic;
    cmmd         : out std_logic_vector(8 downto 0);
    data_rw      : out std_logic_vector(67 downto 0);
    dq           : out std_logic_vector(67 downto 0));

end component rw;

component search is

  generic (
    ---------------------------------------------------------------------------
    -- valid values for generic values
    -- for datawidth = 00, for 68-bit
    --                 01, for 136-bit
    --                 10, for 272-bit
    ---------------------------------------------------------------------------
    datawidth  : std_logic_vector(1 downto 0));
    
            
  port (
    clk2x               : in  std_logic;
    rst                 : in  std_logic;
    user_command        : in  std_logic_vector(5 downto 0);
    gmri                : in  std_logic_vector(2 downto 0);
    sadr                : in  std_logic_vector(2 downto 0);
    ssr_reg_index       : in  std_logic_vector(2 downto 0);
    comparand_reg_index : in  std_logic_vector(3 downto 0);
    data                : in  std_logic_vector(67 downto 0);
    cmmd                : out std_logic_vector(8 downto 0);
    dq                  : out std_logic_vector(67 downto 0);
    cmmdv               : out std_logic;
    s_tr_dq             : out std_logic;
    s_tr_cmdv           : out std_logic;
    s_tr_cmd            : out std_logic;
    ready_s             : out std_logic);

end component search;




-- signals used internally
signal user_command_i : std_logic_vector(5 downto 0);
signal sdata : std_logic_vector(67 downto 0);
signal data : std_logic_vector(67 downto 0);
signal gmri : std_logic_vector(2 downto 0);
signal id_in : std_logic_vector(4 downto 0);
signal sadri : std_logic_vector(2 downto 0);
 
signal s_tr_dq, s_tr_cmdv, s_tr_cmd: std_logic;	-- 3-state enables from search state machine
signal rw_tr_dq, rw_tr_cmdv, rw_tr_cmd: std_logic; -- 3-state enables from rw state machine
signal i_tr_dq, i_tr_cmdv, i_tr_cmd: std_logic; -- 3-state enables from init state machine
signal data_rw, dq, data_back: std_logic_vector(67 downto 0);
signal eot_out_temp, eot_in: std_logic;
signal cmmd: std_logic_vector(8 downto 0);-- Internal command bus, 3-stated internally
signal cmmdv: std_logic; -- Internal command-valid signal, 3-stated internally
signal clk2x: std_logic;
signal ready_i, ready_s, ready_rw: std_logic;-- Internal ready signals from the state machines
signal start_out: std_logic;
signal start_in: std_logic;
--signal lock: std_logic;
signal clkin_i: std_logic;
signal clk: std_logic;

    ---------------------------------------------------------------------------
    -- constants to determine bitwidth
    -- for datawidth = 00, for 68-bit
    --                 01, for 136-bit
    --                 10, for 272-bit
    -- for cascade_depth = any value within bounds, min is "00001"
    -- for mask_depth = 00, for cascade_depth of 1
    --                  01, for cascade_depth of 2 to 8 devices
    --                  10, for cascade_depth of 9 to 31 devices
    ---------------------------------------------------------------------------

-------------------------------------------------------------------------------
-- NOTE : when changing "datawidth", you may also need to change datawidths
-- of the lower modules, INIT and SEARCH. Check your results to ensure
-- things are working as they should.
-------------------------------------------------------------------------------
constant datawidth : std_logic_vector(1 downto 0) := "00";
constant cascade_depth : std_logic_vector(4 downto 0) := "00001";
constant mask_depth : std_logic_vector(1 downto 0) := "00"; 
  
begin  -- top_arch

-- module instantiations
-- Instantiate lower level blocks

clkbufg: IBUFG port map(I => clkin, O => clkin_i);

clk1: clock port map(clkin => clkin_i, 
	    clk2x => clk2x,
		clk => clk,
		rst => dllrst,
		lock => lock,
		phs_l => phs_l,
		clk2xcam => clk2xcam);


rw1: rw port map(user_command => user_command_i, 
		ack => ack, 
		clk => clk,
		rst => rst, 
		cmmd => cmmd, 
		cmmdv => cmmdv, 
		data => data, 
		data_back => dq_out, 
		dq => dq,
		data_rw => data_rw, 
		eot_in => eot, 
		rw_tr_dq => rw_tr_dq, 
		rw_tr_cmdv => rw_tr_cmdv, 
		rw_tr_cmd => rw_tr_cmd,
		ready_rw => ready_rw,
		gmri => gmri,
		id => id_in
                );


init1: init generic map (datawidth => datawidth,
                        cascade_depth => cascade_depth,
						tlsz => mask_depth)
        port map(start => start,
		cmmd => cmmd, 
		dq => dq, 
		cmmdv => cmmdv, 
		clk => clk,
		rst => rst, 
		i_tr_dq => i_tr_dq, 
		i_tr_cmdv => i_tr_cmdv, 
		i_tr_cmd => i_tr_cmd,
		ready_i => ready_i,
		id => id_in,
		id_enable => id_enable,
                last => last
		);


search1: search generic map (datawidth => datawidth)
        port map(user_command => user_command_i, 
		cmmdv => cmmdv, 
		cmmd => cmmd, 
		dq => dq, 
		clk2x => clk2x, 
		rst => rst,
		data => sdata,
		s_tr_dq => s_tr_dq, 
		s_tr_cmdv => s_tr_cmdv, 
		s_tr_cmd => s_tr_cmd,
		ready_s => ready_s,
		gmri => gmri,
		sadr => sadri,
		ssr_reg_index => ssr_reg_index,
		comparand_reg_index => comparand_reg_index);


-- registering of data ports
  
process(clk)
begin
  if (clk'event and clk = '1') then
    data <= rw_in;
    gmri <= gmr_index;
    id_in <= id;

    if (ack = '1') then
      rw_out <= data_rw;
    end if;
  end if;
end process;

process (clk2x)
begin
  if (clk2x'event and clk2x = '1') then
    sdata <= search_in;
    user_command_i <= user_command;
    sadri <= sadr_index;

    if (ssf = '1') then
      search_out <= sadr;
    end if;        
  end if;
end process;

-- combinational assignments

match <= ssf;
match_valid <= ssv;

dq_out <= dq when ((rw_tr_dq and  s_tr_dq and i_tr_dq) = '0') else (others => 'Z');

cmdv_out <= cmmdv when ((rw_tr_cmdv and s_tr_cmdv and i_tr_cmdv) = '0') else 'Z';
cmd_out <= cmmd when ((rw_tr_cmd and  s_tr_cmd and i_tr_cmd) = '0') else (others => 'Z');

ready <= ready_i and ready_s and ready_rw;

end top_arch;
