-------------------------------------------------------------------------------
--    File Name:  test.vhd
--      Version:  1.0
--         Date:  January 24, 2001
--  Description:  functional testbench for Lara Technology CAM Interface
-- Dependencies:  LTI7010, CAM
--
--      Company:  Xilinx
--
--
--   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
--                WHATSOEVER.
--
--                Copyright (c) 2001 Xilinx, Inc.
--                All rights reserved
--
-------------------------------------------------------------------------------

library ieee;
use ieee.std_logic_1164.all;

-------------------------------------------------------------------------------

entity top_tb is

end top_tb;

-------------------------------------------------------------------------------

architecture test of top_tb is

  component top
    port (
      lock                : out   std_logic;
      clkin               : in    std_logic;
      rst                 : in    std_logic;
      dllrst              : in    std_logic;
      user_command        : in    std_logic_vector(5 downto 0);
      rw_in               : in    std_logic_vector(67 downto 0);
      search_in           : in    std_logic_vector(67 downto 0);
      ack                 : in    std_logic;
      ssf                 : in    std_logic;
      ssv                 : in    std_logic;
      eot                 : in    std_logic;
      start               : in    std_logic;
      gmr_index           : in    std_logic_vector(2 downto 0);
      id                  : in    std_logic_vector(4 downto 0);
      sadr                : in    std_logic_vector(21 downto 0);
      sadr_index          : in    std_logic_vector(2 downto 0);
      ssr_reg_index       : in    std_logic_vector(2 downto 0);
      comparand_reg_index : in    std_logic_vector(3 downto 0);
      last                : in    std_logic;
      dq_out              : inout std_logic_vector(67 downto 0);
      search_out          : out   std_logic_vector(21 downto 0);
      match               : out   std_logic;
      match_valid         : out   std_logic;
      ready               : out   std_logic;
      id_enable           : out   std_logic;
      rw_out              : out   std_logic_vector(67 downto 0);
      cmdv_out            : out   std_logic;
      cmd_out             : out   std_logic_vector(8 downto 0);
      clk2xcam            : out   std_logic;
      phs_l               : out   std_logic);
  end component;

  component brecken_ref_leaf_new
    port ( clk2x : in std_logic;
           phs_l : in std_logic; 
           rst_l : in std_logic;

           dq : inout std_logic_vector (67 downto 0);
           cmdv : in std_logic;
           cmd : in std_logic_vector (8 downto 0);
           id : in std_logic_vector (4 downto 0);
           lhi : in std_logic_vector (6 downto 0);
           bhi : in std_logic_vector (2 downto 0);
           fuli : in std_logic_vector ( 6 downto 0);
           test : in std_logic;

           ref_ack_out : out std_logic;
           ref_eot_out : out std_logic;
           ref_ssv_out : out std_logic;
           ref_ssf_out : out std_logic;
           ref_ce_l_out : out std_logic;
           ref_we_l_out : out std_logic;
           ref_oe_l_out : out std_logic;
           ref_ale_l_out : out std_logic;
           ref_sadr_out : out std_logic_vector (21 downto 0);
           ref_full_out : out std_logic;
           ref_fulo_out : out std_logic;
           ref_lho_out : out std_logic;
           ref_bho_out : out std_logic
           );
  end component;



  
  signal clkin               : std_logic;
  signal rst                 : std_logic;
  signal dllrst              : std_logic;
  signal user_command        : std_logic_vector(5 downto 0);
  signal rw_in               : std_logic_vector(67 downto 0);
  signal search_in           : std_logic_vector(67 downto 0);
  signal ack                 : std_logic;
  signal ssf                 : std_logic;
  signal ssv                 : std_logic;
  signal start               : std_logic;
  signal gmr_index           : std_logic_vector(2 downto 0);
  signal id_fixed                 : std_logic_vector(4 downto 0);
  signal sadr1                : std_logic_vector(21 downto 0);
  signal sadr_index          : std_logic_vector(2 downto 0);
  signal ssr_reg_index       : std_logic_vector(2 downto 0);
  signal comparand_reg_index : std_logic_vector(3 downto 0);
  signal last                : std_logic;
  signal dq_out              : std_logic_vector(67 downto 0);
  signal search_out          : std_logic_vector(21 downto 0);
  signal match               : std_logic;
  signal match_valid         : std_logic;
  signal ready               : std_logic;
  signal id_enable           : std_logic;
  signal rw_out              : std_logic_vector(67 downto 0);
  signal cmdv_out            : std_logic;
  signal cmd_out             : std_logic_vector(8 downto 0);
  signal clk2xcam            : std_logic;
  signal phs_l               : std_logic;


  signal cam_rst : std_logic;
  signal eot_out : std_logic;
  signal dlllock : std_logic;
  signal tracker : std_logic_vector(3 downto 0);

  constant CYCLE : time := 20000 ps;      -- this should be in ps (20 ns period)

  
begin  -- test
-- instantiate the CAM model
  cam_inst: brecken_ref_leaf_new
    port map (
      clk2x        => clk2xcam,
      phs_l        => phs_l,
      rst_l        => cam_rst,
      cmdv         => cmdv_out,
      dq           => dq_out,
      cmd          => cmd_out,
      id           => id_fixed,
      lhi          => "0000000",
      bhi          => "000",
      fuli         => "0000000",
      test         => '0',
      ref_ack_out  => ack,
      ref_eot_out  => eot_out,
      ref_ssv_out  => ssv,
      ref_ssf_out  => ssf,
      ref_sadr_out => sadr1);

  DUT: top
    port map (
      lock                => dlllock,
      clkin               => clkin,
      rst                 => rst,
      dllrst              => dllrst,
      user_command        => user_command,
      rw_in               => rw_in,
      search_in           => search_in,
      ack                 => ack,
      ssf                 => ssf,
      ssv                 => ssv,
      eot                 => eot_out,
      start               => start,
      gmr_index           => gmr_index,
      id                  => id_fixed,
      sadr                => sadr1,
      sadr_index          => sadr_index,
      ssr_reg_index       => ssr_reg_index,
      comparand_reg_index => comparand_reg_index,
      last                => last,
      dq_out              => dq_out,
      search_out          => search_out,
      match               => match,
      match_valid         => match_valid,
      ready               => ready,
      id_enable           => id_enable,
      rw_out              => rw_out,
      cmdv_out            => cmdv_out,
      cmd_out             => cmd_out,
      clk2xcam            => clk2xcam,
      phs_l               => phs_l);





--**************************/
-- Begin the test sequence
--**************************/

  testbench: process

-- create some procedures here to manipulate the data
    
    procedure single_read (
      command : in std_logic_vector(5 downto 0);
      addr : in std_logic_vector(14 downto 0)) is
    begin  -- single_read
      if command(4 downto 3) = "00" then
        report "Single read from data array" severity note;
      elsif command(4 downto 3) = "01" then
        report "Single read from mask array" severity note;
      elsif command(4 downto 3) = "10" then
        report "Single read from external SRAM" severity note;
      else
        report "Single read from internal register" severity note;
      end if;

      if ready = '1' then
        user_command <= command; --single read from location 00
      end if;

      rw_in(14 downto 0) <= addr;
      rw_in(67 downto 15) <= '1' & X"0000000000000";
      wait for (5 * CYCLE);
      user_command(0) <= '0'; --de-assert cmdv
      wait for (5 * CYCLE);
    end procedure single_read;

-------------------
    procedure  single_write(
      command: in std_logic_vector(5 downto 0);
      addr : in std_logic_vector(14 downto 0);
      data: in std_logic_vector(67 downto 0)) is


    begin
      if (ready = '1') then
        user_command <= command;
      else
        user_command <= "000000";
      end if;
      gmr_index <= "000";
      rw_in(67 downto 15) <= '1' & X"0000000000000";
      rw_in(14 downto 0) <= addr;

      wait for CYCLE;
      user_command(0) <= '0'; --de-assert cmdv
      rw_in(67 downto 0) <= data;
      wait for (2 * CYCLE);
    end procedure single_write;

------------------
--After calling this task for burst write,
--you must supply the data to be written in subsequent clock cycles

    procedure burst_rw(  
      command : in std_logic_vector (5 downto 0);
      burst_addr : in std_logic_vector (14 downto 0);
      burst_length : in std_logic_vector (8 downto 0)) is

    begin
      if (command (2 downto 1) = "01") then
        report "Burst write starting" severity note;
      else
        report "Burst Read starting " severity note;
      end if;
      if (ready = '1') then
        user_command <= command;
      end if;
      gmr_index <= "000";
      wait for CYCLE;

      rw_in(14 downto 0) <= burst_addr; --burst address = 02
      rw_in (23 downto 15) <= burst_length; --burst lenght = 3
      rw_in (67 downto 24) <= X"00000000000";
      user_command(0) <= '0'; --de-assert cmdv
      wait for CYCLE;

    end procedure burst_rw;






-----------------
    procedure search_68(
      data: in std_logic_vector(67 downto 0);
      ssri : in std_logic_vector(2 downto 0);
      compi: in std_logic_vector(3 downto 0)) is


    begin
      report "Performing a 68-bit search";
      if ready = '1' then
        user_command <= "000101";
      end if;
      sadr_index(2 downto 0) <= "000";
      search_in(67 downto 0) <= data;
      ssr_reg_index <= ssri;
      comparand_reg_index <= compi;
      wait for CYCLE;
      user_command <= "000100";
    end procedure search_68;

----------------
    procedure search_136(
      ssri: in std_logic_vector(2 downto 0);
      compi: in std_logic_vector(3 downto 0)) is

    begin
      report "Performing a 136-bit search";
      if ready = '1' then
        user_command <= "000101";
      end if;

      sadr_index(2 downto 0) <= "000";
      ssr_reg_index <= ssri;
      comparand_reg_index <= compi;
      wait for (CYCLE / 2);

    end procedure search_136;
    

  begin  -- process testbench

    tracker <= "0000";
    rst  <= '0';
    cam_rst <= '0';
    start <= '0';
    last <= '1';
    user_command <= "000000";

    wait for  CYCLE;
    dllrst <= '1'; -- reset the dll
    wait for (5 *  CYCLE); -- hold it for 5 cycles
    dllrst <= '0'; -- relese the dll
    rst <= '1';  -- reset the design.

    wait until dlllock <= '1';
    
    rst <= '0';
    wait for (45 *  CYCLE);
    cam_rst <= '1';
    start <= '1';
    wait for (5 *  CYCLE);
    start <= '0';
    
    wait  for 300 ns;
    wait for (5 *  CYCLE);

    tracker <= "0001";
    -- below is the single write format: 
    --    single_write(user_command, addr, data)
    -- write to GMR1, GMR0 is already initialized
    single_write ("011011", "000" & X"021", X"fffffffffffffffff");

    wait for (5 *  CYCLE);

    --perform a single write to data memory of the CAM location 0
    single_write("000011", "000" & X"000", X"aaaaaaaaaaaaaaaaa"); 

    wait for (5 *  CYCLE);
    tracker <= "0010";
    single_read("011001", "000" & X"038"); --read command register
    wait for (5 *  CYCLE);
    tracker <= "0011";

    -- write to the other general mask registers
    --write to mask loc 02
    single_write("011011", "000" & X"022", X"fffffffffffffffff"); 
    --write to mask loc 03
    single_write("011011", "000" & X"023", X"fffffffffffffffff");
    --write to mask loc 04
    single_write("011011", "000" & X"024", X"fffffffffffffffff"); 
    --write to mask loc 05	
    single_write("011011", "000" & X"025", X"fffffffffffffffff"); 
    --write to mask loc 06
    single_write("011011", "000" & X"026", X"fffffffffffffffff"); 
    --write to mask loc 07
    single_write("011011", "000" & X"027", X"fffffffffffffffff");
    --write to mask loc 08
    single_write("011011", "000" & X"028", X"fffffffffffffffff"); 
    --write to mask loc 09
    single_write("011011", "000" & X"029", X"fffffffffffffffff"); 
    --write to mask loc 10
    single_write("011011", "000" & X"02a", X"fffffffffffffffff"); 
    

    tracker <= "0100";

    -- perform a burst write to the data registers
    -- format for the command is:
    --    burst_rw(usercmd, addr, length);
    burst_rw("100011", "000" & X"000", "000001001");
    wait for (2*  CYCLE);
    rw_in(67 downto 0) <= X"88888888888888888";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"55555555555555555";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"11111111111111111";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"22222222222222222";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"aaaaaaaaaaaaaaaaa";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"bbbbbbbbbbbbbbbbb";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"ccccccccccccccccc";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"ddddddddddddddddd";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"33333333333333333";
    wait for (4 *  CYCLE);


    tracker <= "0101";
    -- try a burst read to see what we get back
    -- we should get back the same data we wrote
    burst_rw("100001", "000" & X"000", "000001001");
    wait for ((4 + (2 * 9)) *  CYCLE);
    wait for (4 *  CYCLE);  -- give some extra time


    tracker <= "0110";
    -- to do a sucessful search, must write to the data mask registers
    -- Burst Write to the mask registers
    burst_rw("101011", "000" & X"000", "000001001");
    wait for (2*  CYCLE);
    rw_in(67 downto 0) <= X"FFFFFFFFFFFFFFFFF";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"FFFFFFFFFFFFFFFFF";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"FFFFFFFFFFFFFFFFF";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"FFFFFFFFFFFFFFFFF";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"FFFFFFFFFFFFFFFFF";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"FFFFFFFFFFFFFFFFF";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"FFFFFFFFFFFFFFFFF";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"FFFFFFFFFFFFFFFFF";
    wait for ( CYCLE);
    rw_in(67 downto 0) <= X"FFFFFFFFFFFFFFFFF";
    wait for (4 *  CYCLE);


    tracker <= "0111";

    -- here is the search test. SADR should be the
    -- address of the data we're looking for.
    -- uncomment the approperate section for the test


    --Search for 68-bit configuration

    search_68(X"55555555555555555", "001", "0001");
    wait for (10 *  CYCLE);
    search_68(X"11111111111111111", "001", "0001");
    wait for (10 *  CYCLE);
    search_68(X"22222222222222222", "001", "0001");
    wait for (10 *  CYCLE);
    -- note the next search should fail since there is no
    -- data with this value written into the testbench
    search_68(X"12345678901234567", "001", "0001");





    
    --Search for 136-bit configuration

--	search_in(67 downto 0) <= X"88888888888888888";
--	@(negedge clkin) search_136("001", "0001");
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"55555555555555555";
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"11111111111111111";
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"22222222222222222";
--	user_command <= "000100;



    --Search for 272-bit configuration

-- 	search_in(67 downto 0) <= X"88888888888888888";
--	@(negedge clkin) search_136("001", "0001");
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"55555555555555555";
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"11111111111111111";
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"22222222222222222";
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"aaaaaaaaaaaaaaaaa";
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"bbbbbbbbbbbbbbbbb";
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"ccccccccccccccccc";
--	wait for (CYCLE / 2);
--	search_in(67 downto 0) <= X"ddddddddddddddddd";
    
--	user_command <= "000100;



    wait for (30 *  CYCLE);
    wait;
    -- end of the simulation
  end process testbench;


-- clock signal generation

  clock_gen: process
  begin  -- process clock gen

    clkin <= '1';
    wait for (CYCLE /2);
    clkin <= '0';
    wait for  (CYCLE / 2);
    
  end process clock_gen;

  
end test;

-------------------------------------------------------------------------------








