/******************************************************************************
*
*    File Name:  top.v
*      Version:  1.1
*         Date:  January 24, 2001
*  Description:  Top Level design for interface with Lara Networks CAM
* Dependencies:  defines.v, init.v, clock.v, rw.v, search.v
*
*      Company:  Xilinx
*
*
*   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
*                WHATSOEVER.
*
*                Copyright (c) 2001 Xilinx, Inc.
*                All rights reserved
*
******************************************************************************/

module top (  lock,clkin, 
              rst, dllrst, 
              user_command, 
              rw_in, 
              rw_out, 
              search_in, 
              search_out, 
              match, 
              match_valid, 
              start, 
              ready, 
              gmr_index, 
              id, 
              id_enable,      
              dq_out,
              ack, 
              ssf, 
              ssv, 
              eot, 
              cmdv_out, 
              cmd_out, 
              sadr,
              sadr_index, 
              clk2xcam,
              phs_l,
              ssr_reg_index,
              comparand_reg_index,
              last
            );


input clkin, rst, dllrst;    //
input [5:0]user_command;  //
input [2:0]gmr_index;    //  
input [4:0]id;    // 
input start;      //  
input [67:0]rw_in, search_in; // Inputs from User Side

input [21:0]sadr;    //
input [2:0]sadr_index;
input ack, ssf, ssv;    // Inputs from CAM
input eot;

inout [67:0]dq_out;    //
    // Bidirectional signals between Virtex/E device and CAM
input last;

output [67:0]rw_out;    //
output [21:0]search_out;  //  
output match, match_valid;  //
output ready, id_enable;  // Outputs to User Side

output phs_l, clk2xcam;  //
output [8:0]cmd_out;    //  
output cmdv_out;    // Outputs to CAM
output lock; // signal which indicates when the dll is locked.
input [2:0]ssr_reg_index;
input [3:0]comparand_reg_index;

reg [5:0]user_command_i;
reg [67:0]sdata;
reg [67:0]data, rw_out;
reg [21:0]search_out;
reg [2:0]gmri;
reg [4:0]id_in;
reg [2:0]sadri;

 
wire s_tr_dq, s_tr_cmdv, s_tr_cmd;    // 3-state enables from search state machine
wire rw_tr_dq, rw_tr_cmdv, rw_tr_cmd;  // 3-state enables from rw state machine
wire i_tr_dq, i_tr_cmdv, i_tr_cmd;    // 3-state enables from init state machine
wire [67:0]data_rw, dq;//, data_back;      
wire [8:0]cmmd;          // Internal command bus, 3-stated internally
wire cmmdv;            // Internal command-valid signal, 3-stated internally
wire clk2x;
wire ready_i, ready_s, ready_rw;      // Internal ready signals from the state machines


assign match = ssf;
assign match_valid = ssv;


//Instantiate lower level blocks

IBUFG clkbufg(.I(clkin), .O(clkin_i));

clock clk1(  .clkin(clkin_i), 
    .clk2x(clk2x),
    .clk(clk),
    .rst(dllrst),
    .lock(lock),
    .phs_l(phs_l),
    .clk2xcam(clk2xcam));


rw rw1(   .user_command(user_command_i), 
    .ack(ack), 
    .clk(clk),
    .rst(rst), 
    .cmmd(cmmd), 
    .cmmdv(cmmdv), 
    .data(data), 
    .data_back(dq_out), 
    .dq(dq),
    .data_rw(data_rw), 
    .eot_in(eot), 
    .rw_tr_dq(rw_tr_dq), 
    .rw_tr_cmdv(rw_tr_cmdv), 
    .rw_tr_cmd(rw_tr_cmd),
    .ready_rw(ready_rw),
    .gmri(gmri),
    .id(id_in));


init init1(  .start(start),
    .cmmd(cmmd), 
    .dq(dq), 
    .cmmdv(cmmdv), 
    .clk(clk),
    .rst(rst), 
    .i_tr_dq(i_tr_dq), 
    .i_tr_cmdv(i_tr_cmdv), 
    .i_tr_cmd(i_tr_cmd),
    .ready_i(ready_i),
    .id(id_in),
    .id_enable(id_enable),
    .last (last)
    );


search search1(  .user_command(user_command_i), 
    .cmmdv(cmmdv), 
    .cmmd(cmmd), 
    .dq(dq), 
    .clk2x(clk2x), 
    .rst(rst),
    .data(sdata),
    .s_tr_dq(s_tr_dq), 
    .s_tr_cmdv(s_tr_cmdv), 
    .s_tr_cmd(s_tr_cmd),
    .ready_s(ready_s),
    .gmri(gmri),
    .sadr(sadri),
    .ssr_reg_index(ssr_reg_index),
    .comparand_reg_index(comparand_reg_index));


//Registers
always @(posedge clk)
begin
  data     <= rw_in;
  gmri     <= gmr_index;
  id_in    <= id;
  
  if (ack==1'b1)
    rw_out <= data_rw;
  else
    rw_out <= rw_out;
end

always @(posedge clk2x)
begin
  sdata <= search_in;
  user_command_i <= user_command;
  sadri <= sadr_index;

  if(ssf==1'b1)
    search_out <= sadr;
  else
    search_out <= search_out;
end


//
assign dq_out   = ((rw_tr_dq && s_tr_dq && i_tr_dq) ==1'b0)?dq: 68'hz;
assign cmdv_out = ((rw_tr_cmdv && s_tr_cmdv && i_tr_cmdv) ==1'b0)? cmmdv : 1'bz;
assign cmd_out  = ((rw_tr_cmd && s_tr_cmd && i_tr_cmd) ==1'b0)? cmmd : 9'hz;
assign ready    = (ready_i && ready_s && ready_rw);

endmodule

















