/******************************************************************************
*
*    File Name:  test.v
*      Version:  1.1
*         Date:  January 24, 2001
*  Description:  functional testbench for Lara Technology CAM Interface
* Dependencies:  LTI7010, CAM
*
*      Company:  Xilinx
*
*
*   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
*                WHATSOEVER.
*
*                Copyright (c) 2001 Xilinx, Inc.
*                All rights reserved
*
******************************************************************************/
`define HALF_CYCLE 1000   //10ns, 100Mhz
`define CYCLE 2000  //20ns, 50Mhz
// glbl.v is the Xilinx global signal simulation file which is included
// with the Xilinx software.
`include "glbl.v"
// this file was the simulated LARA CAM model
`include "brecken_ref_leaf_new.v"
// Xilinx reference design file
`include "top.v"
`timescale 10ps / 10ps

module test; 

parameter high = 1'b1;
parameter low = 1'b0;
parameter id_fixed = 5'b00000;


reg 	clkin, rst, start, cam_rst, phase; 
wire 	ssf, ssv;
reg 	[67:0] rw_in, search_in;
wire 	[21:0] sadr1;
reg 	[5:0] user_command;
wire 	[67:0]dq_out, rw_out; 
wire 	[21:0]search_out;
wire 	[8:0] cmd_out;
wire 	cmdv_out;
reg 	[4:0]id;
reg 	[2:0]gmr_index;
tri0	eot_out, ack;
reg 	[2:0]sadr_index, ssr_reg_index;
reg 	[3:0] comparand_reg_index;
wire 	ready;
reg	last;
wire 	dlllock;
reg     dllrst;

reg 	[3:0]tracker;  // internal tracking signal to figure out where we are



// define some tasks to simplify the simulation 
task single_read;
input [5:0] command;
input [14:0] addr;

   begin
	if(command[4:3] == 2'b00) $display ("Single Read from Data Array");
	else if (command[4:3] == 2'b01) $display ("Single Read from Mask Array");
	else if (command[4:3] == 2'b10) $display ("Single Read from External SRAM");
	else $display ("Single Read from Internal Register");

	if (ready == 1'b1) user_command = command; //single read from location 00
	rw_in[14:0] = addr;
        rw_in[67:15] = 53'h00000000000000;
	#(`CYCLE);
	user_command[0] = 1'b0; //de-assert cmdv
        #(5*`CYCLE);	
  end
endtask


//-----------------
task single_write;
input [5:0] command;
input [14:0] addr;
input [67:0] data;
   begin
	if (ready == 1'b1) user_command = command;
        else user_command = 6'b000000; 
	gmr_index = 3'b000;
        rw_in[67:15] = 54'h00000000000000;
	rw_in[14:0] = addr;
	#(`CYCLE);
	user_command[0] = 1'b0; //de-assert cmdv
//	#(`CYCLE);
	rw_in[67:0] = data;
        #(2 * `CYCLE);
  end	
endtask

//----------------
//After calling this task for burst write,
//you must supply the data to be written in subsequent clock cycles

task burst_rw; 
input [5:0] command;
input [14:0] burst_addr;
input [8:0] burst_length;

  begin
	if(command[2:1] == 2'b01) $display("Burst write at starting address %h",burst_addr);
	else $display("Burst Read at starting address %h",burst_addr);

	if (ready == 1'b1) user_command = command;
	gmr_index = 3'b000;
	#(`CYCLE);
	rw_in[14:0] = burst_addr; //burst address = 02
	rw_in [23:15] = burst_length; //burst lenght = 3
        rw_in [67:24] = 44'h00000000000;
	user_command[0] = 1'b0; //de-assert cmdv
	#(`CYCLE);

  end
endtask

//---------------
task search_68;
input [67:0] data;
input [2:0] ssri;
input [3:0] compi;
  begin
	$display ("Performing a 68-bit search");
	if (ready == 1'b1) user_command = 6'b000101;
	sadr_index[2:0] = 3'b000;
	search_in[67:0] = data;
	ssr_reg_index = ssri;
	comparand_reg_index = compi;
	#(`CYCLE);
	user_command = 6'b000100;
  end
endtask

//--------------
task search_136;
input [2:0] ssri;
input [3:0] compi;
  begin
	$display("Performing a 136-bit search");
	if(ready == 1'b1) user_command = 6'b000101;
	sadr_index[2:0] = 3'b000;
	ssr_reg_index = ssri;
	comparand_reg_index = compi;
	#(`HALF_CYCLE);
  end	
endtask


//------------Begin Testbench-----------------------------------------------

// instantiate the CAM controller module
top top_inst(.lock(dlllock), .clkin(clkin), .rst(rst), .dllrst(dllrst),
             .user_command(user_command), .rw_in(rw_in), .rw_out(rw_out),
             .search_in(search_in), .search_out(search_out),
             .dq_out(dq_out), .ack(ack), .ssf(ssf), .ssv(ssv),
             .eot(eot_out), .cmdv_out(cmdv_out), .cmd_out(cmd_out),
             .sadr(sadr1), .match(match), .match_valid(match_valid),
             .start(start), .ready(ready), .id(id_fixed),
             .gmr_index(gmr_index), .id_enable(id_enable), .phs_l(phs_l),
             .clk2xcam(clk2xcam), .sadr_index(sadr_index), 
             .ssr_reg_index(ssr_reg_index),
             .comparand_reg_index(comparand_reg_index), .last(last) );

// instantiate the CAM model
brecken_ref_leaf_new cam_inst(.clk2x(clk2xcam), .phs_l(phs_l),
                              .rst_l(cam_rst), .cmdv(cmdv_out),
                              .dq(dq_out),.cmd(cmd_out), .id(id_fixed),
                              .lhi({7{low}}), .bhi({3{low}}),
                              .fuli({7{low}}), .test(low),
                              .ref_ack_out(ack), .ref_eot_out(eot_out), 
                              .ref_ssv_out(ssv), .ref_ssf_out(ssf),
                              .ref_sadr_out(sadr1));


//**************************/
// Begin the test sequence
//**************************/


initial begin
        tracker = 4'b0000;
  	rst  = 1'b0;
	cam_rst = 1'b0;
	start = 1'b0;
	last = 1'b1;
	user_command = 6'b000000;
	# `CYCLE;
        dllrst = 1'b1; // reset the dll
        #(5 * `CYCLE) // hold it for 5 cycles
        dllrst = 1'b0; // relese the dll
	rst = 1'b1;  // reset the design.
	# (5 * `CYCLE);
        #65010	// keep the device in reset until the dlls lock.
	rst = 1'b0;
	#(45 * `CYCLE);
	cam_rst = 1'b1;
	start = 1'b1;
	//force test.top_inst.init1.start_out = 1'b1;
        #(5 * `CYCLE);
	start = 1'b0;
	//release test.top_inst.init1.start_out;
 
        #300;
	# (5 * `CYCLE);

        tracker = 4'b0001;
        // below is the single write format: 
        //    single_write(user_command, addr, data)
        // write to GMR1, GMR0 is already initialized
	single_write (6'b011011, 15'h0021, 68'hfffffffffffffffff);
        # (5 * `CYCLE);
        //perform a single write to data memory of the CAM location 0
	single_write(6'b000011, 15'h0000, 68'haaaaaaaaaaaaaaaaa); 

        # (5 * `CYCLE);
        tracker = 4'b0010;
	single_read(6'b011001,15'h0038); //read command register
        # (5 * `CYCLE);
        tracker = 4'b0011;

        // write to the other general mask registers
        //write to mask loc 02
        single_write(6'b011011, 15'h0022, 68'hfffffffffffffffff); 
        //write to mask loc 03
        single_write(6'b011011, 15'h0023, 68'hfffffffffffffffff);
        //write to mask loc 04
        single_write(6'b011011, 15'h0024, 68'hfffffffffffffffff); 
        //write to mask loc 05	
        single_write(6'b011011, 15'h0025, 68'hfffffffffffffffff); 
        //write to mask loc 06
        single_write(6'b011011, 15'h0026, 68'hfffffffffffffffff); 
        //write to mask loc 07
        single_write(6'b011011, 15'h0027, 68'hfffffffffffffffff);
        //write to mask loc 08
        single_write(6'b011011, 15'h0028, 68'hfffffffffffffffff); 
        //write to mask loc 09
        single_write(6'b011011, 15'h0029, 68'hfffffffffffffffff); 
        //write to mask loc 10
        single_write(6'b011011, 15'h002a, 68'hfffffffffffffffff); 
	

        tracker = 4'b0100;

        // perform a burst write to the data registers
        // format for the command is:
        //    burst_rw(usercmd, addr, length);
	burst_rw(6'b100011, 15'h0000, 9'b000001001);
	#(2* `CYCLE);
	rw_in[67:0] = 68'h88888888888888888;
	#(`CYCLE);
	rw_in[67:0] = 68'h55555555555555555;
	#(`CYCLE);
	rw_in[67:0] = 68'h11111111111111111;
	#(`CYCLE);
	rw_in[67:0] = 68'h22222222222222222;
	#(`CYCLE);
	rw_in[67:0] = 68'haaaaaaaaaaaaaaaaa;
	#(`CYCLE);
	rw_in[67:0] = 68'hbbbbbbbbbbbbbbbbb;
	#(`CYCLE);
	rw_in[67:0] = 68'hccccccccccccccccc;
	#(`CYCLE);
	rw_in[67:0] = 68'hddddddddddddddddd;
	#(`CYCLE);
	rw_in[67:0] = 68'h33333333333333333;
	#(4 * `CYCLE);


        tracker = 4'b0101;
        // try a burst read to see what we get back
        // we should get back the same data we wrote
        burst_rw(6'b100001, 15'h0000, 9'b000001001);
        #((4 + (2 * 9)) * `CYCLE)
        #(4 * `CYCLE)  // give some extra time


        tracker = 4'b0110;
        // to do a sucessful search, must write to the data mask registers
        // Burst Write to the mask registers
	burst_rw(6'b101011, 15'h0000, 9'b000001001);
	#(2* `CYCLE);
	rw_in[67:0] = 68'hFFFFFFFFFFFFFFFFF;
	#(`CYCLE);
	rw_in[67:0] = 68'hFFFFFFFFFFFFFFFFF;
	#(`CYCLE);
	rw_in[67:0] = 68'hFFFFFFFFFFFFFFFFF;
	#(`CYCLE);
	rw_in[67:0] = 68'hFFFFFFFFFFFFFFFFF;
	#(`CYCLE);
	rw_in[67:0] = 68'hFFFFFFFFFFFFFFFFF;
	#(`CYCLE);
	rw_in[67:0] = 68'hFFFFFFFFFFFFFFFFF;
	#(`CYCLE);
	rw_in[67:0] = 68'hFFFFFFFFFFFFFFFFF;
	#(`CYCLE);
	rw_in[67:0] = 68'hFFFFFFFFFFFFFFFFF;
	#(`CYCLE);
	rw_in[67:0] = 68'hFFFFFFFFFFFFFFFFF;
	#(4 * `CYCLE);


        tracker = 4'b0111;

        // here is the search test. SADR should be the
        // address of the data we're looking for.
        // uncomment the approperate section for the test


	//Search for 68-bit configuration

	search_68(68'h55555555555555555, 3'b001, 4'b0001);
        #(10 * `CYCLE)
        search_68(68'h11111111111111111, 3'b001, 4'b0001);
        #(10 * `CYCLE)
	search_68(68'h22222222222222222, 3'b001, 4'b0001);
        #(10 * `CYCLE)
        // note the next search should fail since there is no
        // data with this value written into the testbench
	search_68(68'h12345678901234567, 3'b001, 4'b0001);





	
        //Search for 136-bit configuration

/*	search_in[67:0] = 68'h88888888888888888;
	@(negedge clkin) search_136(3'b001, 4'b0001);
	#(`HALF_CYCLE);
	search_in[67:0] = 68'h55555555555555555;
	#(`HALF_CYCLE);
	search_in[67:0] = 68'h11111111111111111;
	#(`HALF_CYCLE);
	search_in[67:0] = 68'h22222222222222222;
	user_command = 6'b000100;

*/

        //Search for 272-bit configuration

/* 	search_in[67:0] = 68'h88888888888888888;
	@(negedge clkin) search_136(3'b001, 4'b0001);
	#(`HALF_CYCLE);
	search_in[67:0] = 68'h55555555555555555;
	#(`HALF_CYCLE);
	search_in[67:0] = 68'h11111111111111111;
	#(`HALF_CYCLE);
	search_in[67:0] = 68'h22222222222222222;
	#(`HALF_CYCLE);
	search_in[67:0] = 68'haaaaaaaaaaaaaaaaa;
	#(`HALF_CYCLE);
	search_in[67:0] = 68'hbbbbbbbbbbbbbbbbb;
	#(`HALF_CYCLE);
	search_in[67:0] = 68'hccccccccccccccccc;
	#(`HALF_CYCLE);
	search_in[67:0] = 68'hddddddddddddddddd;
	
	user_command = 6'b000100;

*/


	#(30 * `CYCLE);
	$stop;

        // end of the simulation
	
end


// clock signal generation

always #`HALF_CYCLE clkin = ~clkin;

initial begin
  clkin = 1'b0;        
end
   

endmodule 






































